/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE10.C 1.1 1996/09/17 12:33:00 NIKH Exp $
 * $Locker: $
 *
 * This is a DOS port of the PlayStation BRender demo "sample10", the
 * CGDC/ECTS/E3 '96 demonsration
 */

/*
 * BRender headers
 */
#include "brender.h"
#include "dosio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample10 prototypes
 */
br_int_8 Sample10_InitWorld(void);
br_int_8 Sample10_RenderFn(void);
br_int_8 Sample10_UpdateFn(void);
br_int_8 Sample10_PadTest(void);

int demoend;

_sample_fn_list Sample10_fn_table = {

        _GenericInitDevice,                     /* init DOS Gfx, BRender */
        Sample10_InitWorld,                      /* init data structures */
        Sample10_RenderFn,                       /* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
        Sample10_UpdateFn,                       /* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
        Sample10_PadTest,                        /* joypad (mouse/keyboard)*/
        _GenericTerminate,                      /* terminate sample (shut down BRender, DOSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample10_fn_table;

int counter = 0, animate = 1, counter1, counter2, counter3, auto_change, frames;
br_scalar xpos, ypos, zpos;
extern br_uint_8 debug;

struct {
    
	char *filename;
	br_uint_16 clut_x;
	br_uint_16 clut_y;
	br_uint_8 palette_trans;
	br_uint_16 tpage_x;
	br_uint_16 tpage_y;
	    
} Bitmaps[] = {

        {"spship.pix", 512, 256+241, 1, 448, 0},
        {"logo.pix", 512, 256+242, 0, 320, 0},
        {"floor_0.pix", 512, 256+240, 1, 576, 0},
	{"plant.pix", 512, 256+244, 0, 640, 0},
        {"brender.pix", 512, 256+243, 0, 320, 256},
	{"cloud.pix", 512, 256+245, 0, 448, 256},
	{"column.pix", 512, 256+246, 1, 512, 256},
    
};

br_actor *pivot0, *floor, *object, *column[6], *pivot1;
br_order_table *primary_order_table;
br_matrix34 *loop;

br_uint_32 *ot_start[2];
br_uint_32 CameraLookAt;

typedef void camera_fn(void);

void CameraReset0(void)
{
        BrMatrix34Identity(&pivot1->t.t.mat);
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-6); zpos = BR_SCALAR(-30);
	CameraLookAt=1;
}
void CameraUpdate0(void)
{
}

void CameraReset1(void)
{
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-7); zpos = BR_SCALAR(8);
	CameraLookAt=1;
}
void CameraUpdate1(void)
{
}
void CameraReset2(void)
{
        camera->t.type = BR_TRANSFORM_MATRIX34;
	CameraLookAt=0;
}
void CameraUpdate2(void)
{
        BrMatrix34Copy(&camera->t.t.mat, loop + ((counter + 170)%180));
        BrMatrix34PreTranslate(&camera->t.t.mat, xpos, ypos, zpos);
}
void CameraReset3(void)
{
        BrMatrix34Identity(&pivot1->t.t.mat);
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-20); zpos = BR_SCALAR(-50);
	CameraLookAt=1;
}
void CameraUpdate3(void)
{
        BrMatrix34RotateY(&pivot1->t.t.mat, BR_ANGLE_DEG(counter1));
}

struct {

	camera_fn *update;
	camera_fn *reset;
	br_uint_32 frames;

} CameraModes[] = {

	{	CameraUpdate0, CameraReset0, 1800,	},
	{	CameraUpdate1, CameraReset1, 1800,	},
	{	CameraUpdate2, CameraReset2, 900,	},
	{	CameraUpdate3, CameraReset3, 1800,	},
};

br_uint_32 NumCameraModes, CurrentCameraMode;

void VBL_cbfn(br_uint_8 db_index)
{
        BrZsSceneRender(root, camera, back_screen);
        BrPixelmapDoubleBuffer(screen, back_screen);
}

/*
 * setup data structures
 */
br_int_8 Sample10_InitWorld(void)
{
        br_pixelmap *pm, *texture, *palette;
	br_model *model;
	br_material *mats[32];
	br_order_table *order_table;
	
	int i;

        pm = BrPixelmapLoad("std.pal");
        if(pm != NULL)
                DOSGfxPaletteSet(pm);
        else
                BR_ERROR0("Couldn't load palette");

        pm = BrPixelmapLoad("shade.tab");
        if(pm != NULL)
                BrTableAdd(pm);
        else
                BR_ERROR0("Couldn't load shade table");

	NumCameraModes = BR_ASIZE(CameraModes);
	CurrentCameraMode = 0;
	frames = CameraModes[CurrentCameraMode].frames;
	auto_change = 1;


	for(i=0; i<BR_ASIZE(Bitmaps); i++) {

                if((pm = BrPixelmapLoad(Bitmaps[i].filename)) == NULL)
			BR_ERROR1("Unable to load '%s' from host", Bitmaps[i].filename);
	
                BrMapAdd(pm);
        }

	/*
	 * load some materials from script
	 */
        i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
	 */
	pivot0 = BrActorAllocate(BR_ACTOR_NONE, NULL);
        pivot0->t.type = BR_TRANSFORM_MATRIX34;
        BrMatrix34Translate(&pivot0->t.t.mat, xpos, ypos, zpos);

	pivot1 = BrActorAdd(pivot0, BrActorAllocate(BR_ACTOR_NONE, NULL));
        pivot1->t.type = BR_TRANSFORM_MATRIX34;
        BrMatrix34Identity(&pivot1->t.t.mat);

	CameraModes[CurrentCameraMode].reset();

	floor = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	model = BrModelLoad("floor.dat");
	
	floor->model = BrModelAdd(model);
        floor->t.type = BR_TRANSFORM_MATRIX34;
        BrMatrix34Translate(&floor->t.t.mat, 0, BR_SCALAR(-5), 0);

	model = BrModelLoad("column.dat");
        model = BrModelAdd(model);

	for(i=0; i<4; i++) {
		column[i] = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
                column[i]->model = model;
                column[i]->t.type = BR_TRANSFORM_MATRIX34;

                BrMatrix34Translate(&column[i]->t.t.mat, BR_SCALAR(18), 0, 0);
                BrMatrix34PostRotateY(&column[i]->t.t.mat, BR_ANGLE_DEG(i*(360/4) + (360/4)/2));
	}

	object = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
        object->model = BrModelLoad("ship.dat");
        object->model = BrModelAdd(object->model);

        object->t.type = BR_TRANSFORM_MATRIX34;

        primary_order_table = BrZsOrderTableAllocate(2048, BR_ORDER_TABLE_LEAVE_BOUNDS, BR_SORT_MIN);
        primary_order_table->min_z = BR_SCALAR(0);
        primary_order_table->max_z = BR_SCALAR(100);

        BrZsActorOrderTableSet(object, BrZsOrderTableAllocate(1024, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_MAX));

#if 0
	/*
	 * prepare logo
	 */
	pm = BrMapFind("logo.pix");
	{
	}
	
	/*
	 * prepare brender
	 */
	pm = BrMapFind("brender.pix");
	{
	}
#endif

	BrZsOrderTablePrimaryEnable(primary_order_table);
	BrZsActorOrderTableSet(root, primary_order_table);

	BrActorAdd(root, pivot0);

	counter = 0, counter1=0, counter2 = 0, counter3 = 0;

        loop = BrMemAllocate(sizeof(br_matrix34) * 360, BR_MEMORY_APPLICATION);

	for(i=0; i<=180; i++) {

		counter += 2;
		if(counter >= 360) counter -= 360;
		counter1 += 4;
		if(counter1 >= 360) counter1 -= 360;

                BrMatrix34Translate(&object->t.t.mat, BR_ADD(BR_MUL(BR_SIN(BR_ANGLE_DEG(counter1)), BR_SCALAR(8)), BR_SCALAR(18)), BR_SIN(BR_ANGLE_DEG(counter1*2)), 0);
                BrMatrix34PostRotateY(&object->t.t.mat, BR_ANGLE_DEG(counter));
                BrMatrix34PreRotateZ(&object->t.t.mat, BR_CONST_DIV(BR_SIN(BR_ANGLE_DEG(counter1)), 10));

                BrMatrix34Copy(loop + i, &object->t.t.mat);
	}

	counter=0, counter1=0;

        return 0;
}

/*
 * BRender Rendering function
 */
br_int_8 Sample10_RenderFn(void)
{
        BrPixelmapFill(back_screen, 0);

        BrZsSceneRenderBegin(root, camera, back_screen);
        BrZsSceneRenderAdd(pivot0);
	BrZsSceneRenderEnd();

        return 0;
}

/*
 * data Update function
 */
br_int_8 Sample10_UpdateFn(void)
{
	br_transform xform;

	counter++;
	if(counter >= 180) counter -= 180;

	counter1 += 2;
	if(counter1 >= 360) counter1 -= 360;
	
	counter2++;
	if(counter2 >= 60) counter2 -= 60;

	if((counter2 % 2) == 0)
		counter3 += 1;
	if(counter3 == 64) counter3 = 0;

	if(animate == 1) {
                BrMatrix34Copy(&object->t.t.mat, loop + counter);
	}

	if(CameraLookAt == 1) {
		BrVector3SetInt(&camera->t.t.look_up.up, 0, 1, 0);
                BrVector3Set(&camera->t.t.look_up.t, 0, 0, 0);
	}

        BrMatrix34Translate(&pivot0->t.t.mat, xpos, ypos, zpos);

	CameraModes[CurrentCameraMode].update();

	if(CameraLookAt == 1) {
		BrActorToActorMatrix34(&xform.t.mat, object, camera->parent);
		BrVector3Sub(&camera->t.t.look_up.look, &xform.t.translate.t, &camera->t.t.look_up.t);
	}

        return 0;
}

/*
 * Pad test
 */
br_int_8 Sample10_PadTest(void)
{
        static br_int_32 mouse_x, mouse_y, prevmouse_x, prevmouse_y;
        static br_boolean mouse_start = BR_FALSE;
        br_uint_32 mouse_buttons;
        int ret = 0, start = 0;

        if(DOSKeyTest(SC_Q, 0, REPT_FIRST_DOWN))
                demoend = -1;

	if(auto_change == 1) {

		frames--;
		if(frames == 0)
                        start = 1;
	}

        if(DOSKeyTest(SC_A, 0, REPT_FIRST_DOWN)) {

	    	animate ^= 1;

	}

        if(DOSKeyTest(SC_C, 0, REPT_FIRST_DOWN)) {

		auto_change ^= 1;

	}

        if(start || DOSKeyTest(SC_SPACE, 0, REPT_FIRST_DOWN)) {

		CurrentCameraMode = (CurrentCameraMode + 1) % NumCameraModes;
		CameraModes[CurrentCameraMode].reset();

		frames = CameraModes[CurrentCameraMode].frames;

	}

        if(DOSKeyTest(SC_C_UP, 0, REPT_AUTO_REPT)) ypos = BR_ADD(ypos,BR_SCALAR(0.5));
        if(DOSKeyTest(SC_C_DOWN, 0, REPT_AUTO_REPT)) ypos = BR_SUB(ypos,BR_SCALAR(0.5));

        if(DOSKeyTest(SC_C_LEFT, 0, REPT_AUTO_REPT)) xpos = BR_SUB(xpos,BR_SCALAR(0.5));
        if(DOSKeyTest(SC_C_RIGHT, 0, REPT_AUTO_REPT)) xpos = BR_ADD(xpos,BR_SCALAR(0.5));

        if(DOSKeyTest(SC_1, 0, REPT_AUTO_REPT)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

        if(DOSKeyTest(SC_0, 0, REPT_AUTO_REPT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

        DOSMouseRead(&mouse_x, &mouse_y, &mouse_buttons);
        if((mouse_x != prevmouse_x) || (mouse_y != prevmouse_y))
                if(mouse_start){
                        xpos = BR_ADD(xpos, BR_SCALAR(mouse_x - prevmouse_x)>>4);
                        if(mouse_buttons & BR_MSM_BUTTONL)
                                zpos = BR_ADD(zpos, BR_SCALAR(mouse_y - prevmouse_y)>>4);
                        else
                                ypos = BR_ADD(ypos, BR_SCALAR(mouse_y - prevmouse_y)>>4);
                } else
                        mouse_start = BR_TRUE;
        prevmouse_x = mouse_x;
        prevmouse_y = mouse_y;

	if(ypos > BR_SCALAR(3)) ypos = BR_SCALAR(3);
	if(ypos < BR_SCALAR(-60)) ypos = BR_SCALAR(-60);

	if(zpos > BR_SCALAR(80)) zpos = BR_SCALAR(80);
	if(zpos < BR_SCALAR(-80)) zpos = BR_SCALAR(-80);

	if(xpos > BR_SCALAR(80)) xpos = BR_SCALAR(80);
	if(xpos < BR_SCALAR(-80)) xpos = BR_SCALAR(-80);

#if 0
	if(PAD_COMPARE(padd, PAD_SELECT)) {

		br_pixelmap *pm;
		br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

		/*
		 * screen capture - useful for checking VRAM
		 */
		pm = BrPixelmapAllocate(back_screen[db_index]->type,
					back_screen[db_index]->width,
					back_screen[db_index]->height,
					NULL,
					0);

		BrPixelmapCopy(pm, back_screen[db_index]);

		DrawSync(0);

		BrPixelmapSave("out.pix", pm);

		BrPixelmapFree(pm);

		while(PAD_COMPARE(PadRead(1), PAD_SELECT));	    
	}
	
	if(debug == 1) {

		FntPrint("\n\n\n\n\n\n\n");
		FntPrint("Camera %d\n", CurrentCameraMode);
		FntPrint("x %d\ny %d\nz %d\n", BrScalarToInt(xpos), BrScalarToInt(ypos), BrScalarToInt(zpos));

		if(auto_change == 1) {
			FntPrint("%3d Auto\n", frames/60);
		}
	}
#endif

	return ret;
}
