/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SUPPORT.C 1.2 1996/09/17 12:40:21 NIKH Exp $
 * $Locker: $
 *
 * PC Generic test harness, common support functions
 */

/*
 * BRender headers
 */
#include "brender.h"
#include "dosio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * generic support functions
 */
#include "support.h"

#include "string.h"

#define PRIMITIVE_HEAP  1024000            /* BRender scratch workspace */

#define TILE_SIZE	256		/* tile size for textures */

/*
 * screen
 */
br_pixelmap *screen;

/*
 *  back screen
 */
br_pixelmap *back_screen;

/*
 * root of sample actor hierarchy
 */
br_actor *root;

/*
 * sample camera
 */
br_actor *camera;

/*
 * sample
 */
br_actor *light;

/*
 * generic setup of DOS Gfx, BRender
 */
br_int_8 _GenericInitDevice(void)
{
	br_material *mat;

	void *primitive_heap;
	int i;

    	/*
	 * init BRender
	 */
	BrBegin();

	primitive_heap = BrMemAllocate(PRIMITIVE_HEAP, BR_MEMORY_APPLICATION);

	/*
         * init PC hardware interface
	 */
        DOSKeyBegin();
        DOSMouseBegin();

#if 0
#ifndef _BACKGROUND
        draw[0].clear_screen = draw[1].clear_screen = 1;
#else
        draw[0].clear_screen = draw[1].clear_screen = 0;
#endif
#endif

        screen = DOSGfxBegin(NULL);
        screen->origin_x = screen->width/2;
        screen->origin_y = screen->height/2;

        back_screen = BrPixelmapMatch(screen, BR_PMMATCH_OFFSCREEN);

	/*
	 * start up zsort renderer
	 */
        BrZsBegin(screen->type, primitive_heap, PRIMITIVE_HEAP);

	/*
	 * setup generic data for samples
	 */
	root = BrActorAllocate(BR_ACTOR_NONE, NULL);
        root->identifier = strdup("main_root");
        root->t.type = BR_TRANSFORM_MATRIX34;
        BrMatrix34Identity(&root->t.t.mat);

	camera = BrActorAdd(root, BrActorAllocate(BR_ACTOR_CAMERA, NULL));
	camera->identifier = "camera";
	((br_camera *)(camera->type_data))->type = BR_CAMERA_PERSPECTIVE;
	((br_camera *)(camera->type_data))->aspect = BR_SCALAR(1.21);
	((br_camera *)(camera->type_data))->hither_z = BR_SCALAR(0.05);
	((br_camera *)(camera->type_data))->yon_z = BR_SCALAR(100);
        ((br_camera *)(camera->type_data))->field_of_view = BR_ANGLE_DEG(45.0);
	
#ifdef _SAMPLE10
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	BrVector3Set(&camera->t.t.look_up.up, 0, BR_SCALAR(1), 0);
	BrVector3Set(&camera->t.t.look_up.t, 0, 0, 0);
#else
        camera->t.type = BR_TRANSFORM_MATRIX34;
        BrMatrix34Translate(&camera->t.t.mat, 0, 0, BR_SCALAR(1));
#endif	

	light = BrActorAdd(root, BrActorAllocate(BR_ACTOR_LIGHT, NULL));
        light->identifier = strdup("main_light");
        light->t.type = BR_TRANSFORM_MATRIX34;
        ((br_light *)light->type_data)->colour = BR_COLOUR_RGB(255, 255, 255);
        BrMatrix34Identity(&light->t.t.mat);

	BrLightEnable(light);
	
	/*
	 * setup order table at root
	 * (generate new bounds, use average of 3 vertices for sorting buckets)
	 */

#ifdef _SAMPLE10
	BrZsActorOrderTableSet(root,
                BrZsOrderTableAllocate(3, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_FIRST_VERTEX));
#else
	BrZsActorOrderTableSet(root,
                BrZsOrderTableAllocate(1023, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_MAX));
#endif

#if 0
	FntLoad(960, 256);				/* font load */
	SetDumpFnt(FntOpen(32, 32, 320, 240, 0, 200)); /* stream open & define */
#endif

	/*
	 * return VSync mode
	 */
	return 0;
}

br_int_8 _GenericTerminate(void)
{
    	/*
	 * terminate Bucket Z Sort Renderer
	 */
    	BrZsEnd();

	/*
         * terminate DOS Gfx support library
	 */
        DOSGfxEnd();

	/*
	 * terminate BRender
	 */
	BrEnd();

        return 0;
}

extern int demoend;

br_int_8 _GenericLoopTest(void)
{
	br_int_8 ret = 0;

#if 0
	if(PAD_COMPARE(padd, PAD_SELECT)) {
		if(PAD_COMPARE(padd, PAD_START)) {
			ret = -1;
		}
	}
#endif

        return demoend;
}

br_int_8 _GenericDoubleBuffer(void)
{
	/*
         * render display list
	 */
        BrPixelmapDoubleBuffer(screen, back_screen);

        return 0;
}

br_uint_32 BR_CALLBACK ActorEnumCbfn(br_actor *mat, void *arg)
{
	(*(br_uint_32 *)arg)++;
    
    	return BrActorEnum(mat, ActorEnumCbfn, arg);
}


